/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#include "operator/operator.pmh"

#define FE_SSO_DEBUG	FALSE

namespace fe
{

void SurfaceSampleOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("attribute");
	catalog<String>("attribute","label")="Attribute";

	catalog< sp<Component> >("Input Surface");

	catalog< sp<Component> >("Source Surface");
}

void SurfaceSampleOp::handle(Record& a_rSignal)
{
#if FE_SSO_DEBUG
	feLog("SurfaceSampleOp::handle\n");
#endif

	const String attribute=catalog<String>("attribute");

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessorI> spOutputPoint;
	if(!access(spOutputPoint,spOutputAccessible,e_point,e_position)) return;

	sp<SurfaceAccessorI> spOutputAttribute;
	if(!access(spOutputAttribute,spOutputAccessible,e_point,attribute)) return;

	sp<SurfaceAccessibleI> spSourceAccessible;
	if(!access(spSourceAccessible,"Source Surface")) return;

	sp<SurfaceI> spSource;
	if(!access(spSource,spSourceAccessible)) return;
	spSource->setRefinement(0);

	sp<SurfaceAccessorI> spSourceVertices;
	if(!access(spSourceVertices,spSourceAccessible,
			e_primitive,e_vertices)) return;

	sp<SurfaceAccessorI> spSourceAttribute;
	if(!access(spSourceAttribute,spSourceAccessible,e_point,attribute)) return;

	const U32 pointCount=spOutputPoint->count();
	for(U32 pointIndex=0;pointIndex<pointCount;pointIndex++)
	{
		const SpatialVector point=spOutputPoint->spatialVector(pointIndex);
		sp<SurfaceI::ImpactI> spImpact=spSource->nearestPoint(point);
		if(spImpact.isValid())
		{
			const I32 primitiveIndex=spImpact->primitiveIndex();
			const SpatialBary bary=spImpact->barycenter();

			//* HACK presumes triangles, and in same vertex order
			const I32 p0=spSourceVertices->integer(primitiveIndex,0);
			const I32 p1=spSourceVertices->integer(primitiveIndex,1);
			const I32 p2=spSourceVertices->integer(primitiveIndex,2);

			//* TODO other types than Real
			const Real value0=spSourceAttribute->real(p0);
			const Real value1=spSourceAttribute->real(p1);
			const Real value2=spSourceAttribute->real(p2);

			const Real value=value0*bary[0]+value1*bary[1]+
					value2*(1.0-bary[0]-bary[1]);

//			feLog("%d/%d point %s prim %d bary %s",
//					pointIndex,pointCount,c_print(point),
//					primitiveIndex,c_print(bary));
//			feLog(" p %d %d %d val %.6G %.6G %.6G -> %.6G\n",
//					p0,p1,p2,value0,value1,value2,value);

			spOutputAttribute->set(pointIndex,value);
		}
	}

#if FE_SSO_DEBUG
	feLog("SurfaceSampleOp::handle done\n");
#endif
}

} // namespace
